<?php

namespace SGalinski\SgMail\ViewHelpers;

/***************************************************************
 *  Copyright notice
 *
 *  (c) sgalinski Internet Services (http://www.sgalinski.de)
 *
 *  All rights reserved
 *
 *  This script is part of the TYPO3 project. The TYPO3 project is
 *  free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 3 of the License, or
 *  (at your option) any later version.
 *
 *  The GNU General Public License can be found at
 *  http://www.gnu.org/copyleft/gpl.html.
 *
 *  This script is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  This copyright notice MUST APPEAR in all copies of the script!
 ***************************************************************/

use TYPO3\CMS\Fluid\Core\ViewHelper\AbstractConditionViewHelper;

/**
 * ExtendedIfViewHelper
 */
class ExtendedIfViewHelper extends AbstractConditionViewHelper {

	/**
	 * Initializes the "then" and "else" arguments
	 */
	public function initializeArguments() {
		parent::initializeArguments();
		$this->registerArgument('condition', 'boolean', 'Condition expression conforming to Fluid boolean rules', FALSE, FALSE);
		$this->registerArgument('or', 'boolean', 'Condition expression conforming to Fluid boolean rules', FALSE, FALSE);
		$this->registerArgument('or2', 'boolean', 'Condition expression conforming to Fluid boolean rules', FALSE, FALSE);
		$this->registerArgument('or3', 'boolean', 'Condition expression conforming to Fluid boolean rules', FALSE, FALSE);
		$this->registerArgument('or4', 'boolean', 'Condition expression conforming to Fluid boolean rules', FALSE, FALSE);
		$this->registerArgument('and', 'boolean', 'Condition expression conforming to Fluid boolean rules', FALSE, TRUE);
		$this->registerArgument('and2', 'boolean', 'Condition expression conforming to Fluid boolean rules', FALSE, TRUE);
		$this->registerArgument('and3', 'boolean', 'Condition expression conforming to Fluid boolean rules', FALSE, TRUE);
		$this->registerArgument('and4', 'boolean', 'Condition expression conforming to Fluid boolean rules', FALSE, TRUE);
		$this->registerArgument('negate', 'boolean', 'Condition expression conforming to Fluid boolean rules', FALSE, FALSE);
	}
	/**
	 * renders <f:then> child if $condition or $or is true, otherwise renders <f:else> child.
	 *
	 * @return string
	 */
	public function render() {
		if ($this->evaluateConditionFunctionExists()) {
			return parent::render();
		}

		// @todo Remove the following part if only > 7.5 support.
		$conditionResult = ($this->arguments['condition'] || $this->arguments['or'] || $this->arguments['or2'] || $this->arguments['or3'] || $this->arguments['or4'])
						   && $this->arguments['and'] && $this->arguments['and2'] && $this->arguments['and3'] && $this->arguments['and4'];
		if ($this->arguments['negate'] ? !$conditionResult : $conditionResult) {
			return $this->renderThenChild();
		} else {
			return $this->renderElseChild();
		}
	}

	/**
	 * This method decides if the condition is TRUE or FALSE. It can be overriden in extending viewhelpers to adjust functionality.
	 *
	 * @param array $arguments ViewHelper arguments to evaluate the condition for this ViewHelper, allows for flexiblity in overriding this method.
	 * @return bool
	 */
	static protected function evaluateCondition($arguments = NULL) {
		$conditionResult = (
				isset($arguments['condition']) && $arguments['condition'] ||
				isset($arguments['or']) && $arguments['or'] ||
				isset($arguments['or2']) && $arguments['or2'] ||
				isset($arguments['or3']) && $arguments['or3'] ||
				isset($arguments['or4']) && $arguments['or4']
			) && isset($arguments['and']) && $arguments['and'] &&
			isset($arguments['and2']) && $arguments['and2'] &&
			isset($arguments['and3']) && $arguments['and3'] &&
			isset($arguments['and4']) && $arguments['and4'];

		return isset($arguments['negate']) && $arguments['negate'] ? !$conditionResult : $conditionResult;
	}

	/**
	 * Returns true, if the function evaluateCondition exists in the AbstractConditionViewHelper class.
	 *
	 * @deprecated Remove this function if only > 7.5 support.
	 * @return boolean
	 */
	protected function evaluateConditionFunctionExists() {
		return method_exists('TYPO3\CMS\Fluid\Core\ViewHelper\AbstractConditionViewHelper', 'evaluateCondition');
	}
}

?>
